/*
  S60Bible A Bible Reader for SymbianOS
  Copyright (C) 2004 Yohanes Nugroho

  Original work (PalmBible and Bible+):
  Copyright (C) 2001  Ho Chuen Poon
  Modifications (C) 2003 Yih-Chun Hu

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

  Yohanes Nugroho (yohanes@opensource.or.id)
  Kp Areman RT 09/08 No 71 Desa Tugu Cimanggis Depok 16951 Indonesia
*/



/*
 * Filename: Decoder.cpp
 * Description: Bible decoder
 * Author: Yohanes Nugroho
 * Ported from palmbible plus (http://palmbibleplus.sourceforge.net)
 * Creation Date:
 * Last Update: 15/Feb/2004
 * Revisions:
 */
#ifdef USE_LIB_C
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "usermem.h"
#else
#include <e32std.h>
#endif

#include "Decoder.h"

Decoder::Decoder(VersionInfo *vinfo, UInt16 version)
{


  bibleversion = vinfo;
  this->version = version;
  this->dbase = vinfo->getDatabase();
  wordInfo = NULL;
  byteacc = NULL;
  wordReader = NULL;

}

void Decoder::ConstructL()
{
  char *s;
  recordIndex = bibleversion->GetWordRecordIndex(version);
  totalRecord = bibleversion->GetWordTotalRecord(version);
  sepChar = bibleversion->GetSepChar();
  //	sepChar  = ' ';
  byteShifted = bibleversion->GetByteShifted(version);

  //	byteShifted  = 0;

#ifdef USE_LIB_C
  wordReader = new FullReader(dbase, version, recordIndex + 1,
			      recordIndex + totalRecord);
#else
  wordReader = new (ELeave) FullReader(dbase, version, recordIndex + 1,
				       recordIndex + totalRecord);
  CleanupStack::PushL(wordReader);
#endif

  s = (char *) dbase->DmxLockVersionRecord(version, recordIndex);
  //wordInfo =  (WordInfoType *) DmxLockVersionRecord(version, recordIndex);
  UInt16 totalIndexes;
  int idx = 0;
  Mem::Copy(&totalIndexes, s + idx, 2);
  idx += 2;

#ifdef USE_LIB_C
  wordInfo = (WordInfoType *) User::AllocL(sizeof(WordInfoType));
#else
  wordInfo = (WordInfoType *) User::AllocLC(sizeof(WordInfoType));
#endif

  wordInfo->totalIndexes = fixs(totalIndexes);
  wordInfo->wordIndex = (WordIndexType *)User::AllocL(sizeof(WordIndexType)*wordInfo->totalIndexes);
#ifndef USE_LIB_C
  CleanupStack::Pop(wordInfo);
  CleanupStack::Pop(wordReader);
#endif

  //Reverse endiannes
  for (int i=0; i<wordInfo->totalIndexes; i++){
    UInt16 wordLength, totalWord;
    wordLength = 0;
    totalWord = 0;
    Mem::FillZ(&wordInfo->wordIndex[i], sizeof(WordIndexType));
    Mem::Copy(&wordLength, s + idx, 2);
    idx += 2;
    Mem::Copy(&totalWord, s + idx, 2);
    idx += 2;
    wordInfo->wordIndex[i].totalWord = fixs(totalWord);
    wordInfo->wordIndex[i].wordLength = fixs(wordLength);
    Mem::Copy(&(wordInfo->wordIndex[i].compressed), s+idx, 1);
    idx += 1;
    Mem::Copy(&(wordInfo->wordIndex[i].nothing), s+idx, 1);
    idx += 1;
  }
  byteacc = (UInt32*)User::Alloc(sizeof(UInt32)*(wordInfo->totalIndexes+1));
  UInt32 totalByteAcc = 0;
  byteacc[0] = 0;
  for (int i=1; i<=wordInfo->totalIndexes; i++){
    int totalWord = wordInfo->wordIndex[i-1].totalWord;
    int wordLen = wordInfo->wordIndex[i-1].wordLength;
    totalByteAcc += totalWord * wordLen;
    byteacc[i] = totalByteAcc;
  }	
}

void Decoder::getWordIndexLen()
{
  UInt32 totalByteAcc;
  UInt16 totalIndexes, relNum, totalWord, i;

  bool compressed;

  totalIndexes = decWordInfo->totalIndexes;

  decWordIndex = 0;
  decWordLen = 0;

  totalByteAcc = 0;
  totalWord = 0;

  compressed = false;

  relNum = decWordNum - 1;
#if 0
  UInt16 wordLen;
  for (i=0; i<totalIndexes; i++){
    totalWord = decWordInfo->wordIndex[i].totalWord;
    wordLen = decWordInfo->wordIndex[i].wordLength;
    compressed = decWordInfo->wordIndex[i].compressed;
    if (relNum < totalWord){
      decWordLen = wordLen;
      decWordIndex = totalByteAcc + relNum * wordLen;
      i = totalIndexes; /*break*/
    } else {
      totalByteAcc += totalWord * wordLen;
      relNum = (UInt16) (relNum - totalWord);
    }
  }
#endif
#if 1
  compressed = decWordInfo->wordIndex[totalIndexes-1].compressed;
  for (i=0; i<totalIndexes; i++){
    totalWord = decWordInfo->wordIndex[i].totalWord;
    if (relNum < totalWord){
      compressed = decWordInfo->wordIndex[i].compressed;
      decWordLen = decWordInfo->wordIndex[i].wordLength;
      decWordIndex = byteacc[i] + relNum * decWordLen;
      i = totalIndexes; /*break*/
    } else {
      relNum = (UInt16) (relNum - totalWord);
    }
  }
#endif
  decCompressed = compressed;
}

void Decoder::beginVerse()
{
  UInt32 compStart;
  UInt32 shiftLookup[] = { 0, 3, 2, 1 };

  if (decByteShifted) {
    compStart = decVerseStart * 7 / 4;
    decShift = (UInt16) shiftLookup[decVerseStart * 7 % 4];

  } else {
    compStart = decVerseStart * 2;
    decShift = 0;
  }

  decVerseReader->ReadBegin(compStart);

  if (decByteShifted) {
    switch (decShift) {
      case 1:
	decValueBuffer[1] = decVerseReader->ReadChar();
	break;
      case 2:
	decValueBuffer[2] = decVerseReader->ReadChar();
	break;
      case 3:
	decValueBuffer[2] = decVerseReader->ReadChar();
	break;
      default:
	;
    }
  }

}

UInt16 Decoder::readVerse()
{
  UInt16 shiftLookup[] = { 10, 4, 6, 8 };
  UInt32 value;

  if (decByteShifted) {
    switch (decShift) {
      case 0:
	decValueBuffer[0] = decVerseReader->ReadChar();
	decValueBuffer[1] = decVerseReader->ReadChar();
	decValueBuffer[2] = 0;
	break;
      case 1:
	decValueBuffer[0] = decValueBuffer[1];
	decValueBuffer[1] = decVerseReader->ReadChar();
	decValueBuffer[2] = decVerseReader->ReadChar();
	break;
      case 2:
	decValueBuffer[0] = decValueBuffer[2];
	decValueBuffer[1] = decVerseReader->ReadChar();
	decValueBuffer[2] = decVerseReader->ReadChar();
	break;
      case 3:
	decValueBuffer[0] = decValueBuffer[2];
	decValueBuffer[1] = decVerseReader->ReadChar();
	decValueBuffer[2] = 0;
	break;
      default:
	;
    }

    value = decValueBuffer[0] << 16 | decValueBuffer[1] << 8 |
      decValueBuffer[2];
    value = value >> shiftLookup[decShift];
    value = value & 0x3FFF;
    decShift++;

    if (decShift == 4) {
      decShift = 0;
    }

    if (value > 0x3FF0) {
      value |= 0xC000;
    }

    decWordNum = (UInt16)value;

  } else {
    decWordNum = decVerseReader->ReadChar() * 256 +
      decVerseReader->ReadChar();

  }
  return decWordNum;
}

void Decoder::endVerse()
{
  decVerseReader->ReadEnd();
}

void Decoder::beginWord()
{
  //decCharBuffer = (char *) User::AllocL(64);
  //Mem::FillZ(decCharBuffer, 64);
  int i;
  for (i=0; i<64; i++) decCharBuffer[i] = 0; 
}

void Decoder::readWord()
{
  UInt16 i;
  char charValue;

  decWordReader->ReadBegin(decWordIndex);
  for (i = 0; i < decWordLen; i++) {
    charValue = decWordReader->ReadChar();
    decCharBuffer[i] = charValue;
  }
  decCharBuffer[i] = 0;
  decWordReader->ReadEnd();
}

void Decoder::endWord()
{
  //delete decCharBuffer;
}

void Decoder::checkCompressedWordNum()
{
  UInt16 numValue, i;

  if (decWordNum < 0xFFF0) {
    getWordIndexLen();
    if (decCompressed) {
      decRepeat = decWordLen / 2;
      decWordReader->ReadBegin(decWordIndex);
      for (i = 0; i < decRepeat; i++) {
	numValue = decWordReader->ReadChar() * 256
	  + decWordReader->ReadChar();
	decCompressedBuffer[i] = numValue;
      }
      decWordReader->ReadEnd();
    } else {
      decRepeat = 1;
      decCompressedBuffer[0] = decWordNum;
    }
  } else {
    decRepeat = 1;
    decCompressedBuffer[0] = decWordNum;
  }

  decCompressedIndex = 0;
}

void Decoder::readCompressed()
{
  decWordNum = decCompressedBuffer[decCompressedIndex];
  if (decByteShifted && decWordNum > 0x3FF0) {
    decWordNum |= 0xC000;
  }
  decCompressedIndex++;
}

Decoder::~Decoder()
{
  dbase->DmxUnlockVersionRecord(version, recordIndex);
  User::Free(wordInfo->wordIndex);
  User::Free(wordInfo);
  User::Free(byteacc);
  delete wordReader;
}


void Decoder::DecodeVerseType(char *verse,
			      FullReader * recordReader,
			      UInt32 verseStart, UInt16 verseLength, int type)
{
  UInt16 i, j;
  bool addWord;

  //if (verseStart>10) verseStart++;

  if (type==versTextType) {
    addWord = true;
  } else {
    addWord = false;
  }

  decSepChar = sepChar;
  decByteShifted = byteShifted;

  decWordInfo = wordInfo;
  decWordReader = wordReader;
  decVerseReader = recordReader;

  decVerseStart = verseStart;

  decLinePos = 0;
  decCharPos = 0;
  beginWord();

  beginVerse();
  decLineBuffer = verse;

  for (i = 0; i < verseLength; i++) {
    readVerse();
    checkCompressedWordNum();
    for (j = 0; j < decRepeat; j++) {
      readCompressed();
      //addWord = true;

      if (decWordNum==bookTextType || decWordNum==chapTextType ||
	  decWordNum==descTextType || decWordNum==versTextType){
	if (decWordNum==type) {
	  addWord = true;
	} else {
	  addWord = false;
	}
      }

      if (addWord) {
	getWordIndexLen();
	if (decCharPos + decWordLen <
	    searchBufferSize - 1) {
	  readWord();
	  if (decWordLen == 1 && decCharPos > 3
	      && decSepChar != 0) {
	    switch (decCharBuffer[0]) {
	      case '.':
	      case ',':
	      case '?':
	      case '!':
	      case ';':
	      case ':':
	      case '-':
		decCharPos--;
		break;
	    }
	  }
	  if (decSepChar != 0) {

	    decCharBuffer
	      [decWordLen] =
	      decSepChar;
	    decWordLen++;
	    decCharBuffer
	      [decWordLen] =
	      0;
	  }

	  my_strcpy(&verse[decCharPos],
		    decCharBuffer);
						  
	  decCharPos = (UInt16)(decCharPos +
				decWordLen);

	}
      }
    }

  }
  verse[decCharPos] = 0;
  endVerse();
  endWord();
}

void Decoder::DecodeVerseAsString(char *verse,
				  FullReader * recordReader,
				  UInt32 verseStart, UInt16 verseLength)

{
  DecodeVerseType(verse, recordReader, verseStart, verseLength, versTextType);
}


/*formatted string*/
void Decoder::DecodeVerseTypeF(char *verse, UInt16 *linepos,
			       UInt16 *lineformat, int *linecount,
			       FullReader * recordReader,
			       UInt32 verseStart, UInt16 verseLength)
{
  UInt16 i, j;
  bool addWord;


  addWord = true;

  decSepChar = sepChar;
  decByteShifted = byteShifted;

  decWordInfo = wordInfo;
  decWordReader = wordReader;
  decVerseReader = recordReader;

  decVerseStart = verseStart;

  decLinePos = 0;
  decCharPos = 0;
  beginWord();

  beginVerse();
  decLineBuffer = verse;

  *linecount = 0;

  for (i = 0; i < verseLength; i++) {
    readVerse();
    checkCompressedWordNum();
    for (j = 0; j < decRepeat; j++) {
      readCompressed();

      if (decWordNum==bookTextType || decWordNum==chapTextType ||
	  decWordNum==descTextType || decWordNum==versTextType){
	lineformat[*linecount]=decWordNum;
	linepos[*linecount]=decCharPos;
	(*linecount)++;
      }

      getWordIndexLen();
      if (decCharPos + decWordLen <
	  searchBufferSize - 1) {
	readWord();
	if (decWordLen == 1 && decCharPos > 3
	    && decSepChar != 0) {
	  switch (decCharBuffer[0]) {
	    case '.':
	    case ',':
	    case '?':
	    case '!':
	    case ';':
	    case ':':
	    case '-':
	      decCharPos--;
	      break;
	  }
	}
	if (decSepChar != 0) {
	  decCharBuffer [decWordLen] = decSepChar;
	  decWordLen++;
	  decCharBuffer [decWordLen] = 0;
	}

	my_strcpy(&verse[decCharPos],
		  decCharBuffer);
						  
	decCharPos = (UInt16)(decCharPos +
			      decWordLen);

      }
    }

  }
  verse[decCharPos] = 0;
  endVerse();
  endWord();
}

