/*
    S60Bible A Bible Reader for SymbianOS
    Copyright (C) 2004 Yohanes Nugroho

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Yohanes Nugroho (yohanes@opensource.or.id)
    Kp Areman RT 09/08 No 71 Desa Tugu Cimanggis Depok 16951 Indonesia
*/

#include "log.h"
#include "gdi.h"
#include "Viewer.h"
#include "CStringList.h"
#include  <eikenv.h>
#include <eikappui.h>
//#include <aknutils.h>
#include <baclipb.h>
#include <hal.h>
#include <charconv.h>
#include "Reader.h"
#include "VerseInfo.h"
#include "CStringList.h"
#include "greek.h"
#include "hebrew.h"
//#include <S80Bible.rsg>

Viewer::Viewer() : iVerseHead(NULL), iVerseTail(NULL),
	iStartChapter(1), iStartVerse(1), iStartLine(0), iBookList(NULL),
	CurrentEncoding(KCharacterSetIdentifierIso88591),
	iLastFont(-1),
	iLastEnc(-1)	
{
   	iArea = TRect(0,0,0,0);
    	iFont = CEikonEnv::Static()->DenseFont();
        widths = new (ELeave)CArrayFixFlat<TInt>(50);
		
	iColors[EVerseText] = KRgbBlack;
	iColors[EVerseBackground]=KRgbWhite;
	iColors[EVerseNumber] = KRgbBlue;
	iColors[EHeaderText] = KRgbBlue;
	iColors[EHeaderFrame] = KRgbBlue;
	iColors[EHeaderBG] = TRgb(206, 206, 255);		
		
}

Viewer::~Viewer()
{
	CEikonEnv::Static()->ReleaseScreenFont(iCustomFont);
	ClearVerse();
	delete iBookList;
	delete iBookmarkList;
	delete iReader;
        delete widths;
	delete iCharacterConverter;
}

void Viewer::SetColor(ETextObject aTextObject, TRgb aColor)
{
	iColors[aTextObject] = aColor;
}

TRgb Viewer::GetColor(ETextObject aTextObject) const
{
	return iColors[aTextObject];
}

void Viewer::CopyToClipboardL(int count)
{
	RFs fsSession;
	fsSession.Connect();
	CClipboard* cb = CClipboard::NewForWritingLC(fsSession);
	CPlainText* plaintext = CPlainText::NewL();

	CleanupStack::PushL(plaintext);

	VerseInfo *v = GetVerseHead();

	while (v) {
		if (v->chapter==iStartChapter && v->verse==iStartVerse) {
			break;
		}
		v = v->next;
	}

        //this should not happen
        if (!v) return;

	TBuf<120> strversenum;
	char *s = BookName();
	TBuf<256> stat;
	MyStringToDes(s, stat);
	User::Free(s);
	//designed to be able to copy more than one verse
	//but can't be done from the UI yet
	for (int i = 0; i<count; i++){
		for (int j = 0; j<v->lines->Count(); j++) {
			if (j== 0) {
				strversenum.Zero();
				_LIT(CopyFormat,"%S %d:%d ");
				strversenum.Format(CopyFormat, &stat, v->chapter,
					v->verse);
				plaintext->InsertL(plaintext->DocumentLength(),
					strversenum);
			}
			plaintext->InsertL(plaintext->DocumentLength(), _L(" "));
			plaintext->InsertL(plaintext->DocumentLength(), v->lines->At(j));
		}
		v = v->next;
		if (!v) break;
	}

	plaintext->CopyToStoreL(cb->Store(), cb->StreamDictionary(), 0,
		plaintext->DocumentLength());
	cb->CommitL();
	CleanupStack::PopAndDestroy(plaintext);
	CleanupStack::PopAndDestroy(cb);
	fsSession.Close();
}

void Viewer::ConstructL(RFs &aFs)
{
	iFs = aFs;
	iReader = NULL;
/*	
	TInt id = 100;

	TBuf<50> res;
	
	//Todo: enumerate all .ttf on the directory
	TInt x = CEikonEnv::Static()->ScreenDevice()->AddFile(_L("E:\\System\\Apps\\S60Bible\\Vera.ttf"), id);
	res.Format(_L("Addfile: %d"), x);
	Log(res);
	id = 101;
	//Todo: enumerate all .gdr on the directory
	x = CEikonEnv::Static()->ScreenDevice()->AddFile(_L("E:\\System\\Apps\\S60Bible\\cu.gdr"), id);
	res.Format(_L("Addfile: %d"), x);
	Log(res);
*/
    	iBookmarkList = CStringList::NewL(CEikonEnv::Static()->FsSession());

    	//iReader = new (ELeave) Reader();
	_LIT(EMPTY, "");
        //iBibleName = EMPTY;
//   	TRAPD(error, iReader->ConstructL(iFs, EMPTY));

	iCurrentBook = EMPTY;

//    	if (error!=KErrNone) {
  // 		delete iReader;
   //		iReader = NULL;
   //		return;
   //	}
	//iReader->SeekVerse(1, (UInt16)iReader->GetFirstBook(), 1, 1);
	//FillBookList();
      iFs.Connect();
//      iCharacterConverter = CCnvCharacterSetConverter::NewLC();
	      iCharacterConverter = CCnvCharacterSetConverter::NewL();
      iConverterAvailability = iCharacterConverter->PrepareToConvertToOrFromL(CurrentEncoding, iFs);
	isGreek = EFalse;
	isHebrew = EFalse;
//      CleanupStack::Pop(iCharacterConverter);      
}


void Viewer::AddToBookmark()
{
	VerseInfo *v = GetVerseHead();

	while (v) {
		if (v->chapter==iStartChapter && v->verse==iStartVerse) {
			break;
		}
		v = v->next;
	}

	TBuf<120> strversenum;
	TBuf<200> bookmarkLine;
	char *s = BookName();
	TBuf<256> stat;
	MyStringToDes(s, stat);
	User::Free(s);
	if (v->lines->Count()==0) return;
	strversenum.Zero();
	_LIT(KFormat,"\t%S %d:%d\t ");
	strversenum.Format(KFormat, &stat, v->chapter, v->verse);
	bookmarkLine = strversenum;
	bookmarkLine.Append(v->lines->At(0));
	iBookmarkList->AddStringL(bookmarkLine);
}

void Viewer::LoadBookmarkL()
{
	iBookmarkList->LoadFromFileL(iBookmarkFile);
}

void Viewer::SaveBookmarkL()
{
	iBookmarkList->SaveToFileL(iBookmarkFile);
}

void Viewer::DelBookmark(int idx)
{
	iBookmarkList->Content()->Delete(idx);
}

void Viewer::DelAllBookmark()
{
	iBookmarkList->Content()->Reset();
}

CDesCArray* Viewer::GetBookmark()
{
	return iBookmarkList->Content();
}

void Viewer::FillBookList()
{
	if (!IsReady()) return;
	if (iBookList) {
		delete iBookList;
		iBookList = 0;
	}
	iBookList = new (ELeave) CDesCArrayFlat(4);
	int booknum = iReader->GetFirstBook();
	int idx = 0;
	do {
		iBookNumberList[idx++] = booknum;
		int chapter_count, verse_count;
		char *tbookname = iReader->GetBookInfo(booknum, chapter_count, verse_count);
		TBuf<128> bookname;
		MyStringToDes(tbookname, bookname);
		User::Free(tbookname);
		//TBuf<256> bookinfo;		
		iBookList->AppendL(bookname);
		booknum = iReader->GetNextBook(booknum);
	} while (booknum!=-1);
}

CDesCArray* Viewer::GetBookList()
{
	if (!iBookList) FillBookList();
	return iBookList;
}

void Viewer::ClearBookList()
{
	if (iBookList) {
		delete iBookList;
		iBookList = NULL;
	}
}


void Viewer::CalcScrollLen()
{
	iScrollLen = ((iArea.Height()-20-GetFont()->HeightInPixels()) /
		(GetFont()->HeightInPixels()));
}


void Viewer::SetArea(const TRect &aRect)
{
	iArea = aRect;
	CalcScrollLen();
}

TBool Viewer::ChangeBibleL(const TDesC &filename)
{
	int tmp_b = -1, tmp_s=1, tmp_c=1;
	//save current position for the new bible
	Log(_L("ChangeBibleL"));
	if (iReader) {
		Log(_L("Delete Old IReader"));
		tmp_b = iReader->BookNumber();
		tmp_s = iStartVerse;
		tmp_c = iStartChapter;
		ClearVerse();
		delete iReader;
		iReader = NULL;
	}
	Log(_L("Creating IReader"));
	
	
	iReader = new (ELeave) Reader();
	Log(_L("Construct"));
	
	TRAPD(error, iReader->ConstructL(iFs, filename));
	Log(_L("Check Err"));
   	if (error!=KErrNone) {
   		delete iReader;
   		iReader = NULL;
   		return EFalse;
   	}
	Log(_L("OK"));
	
   	//FillBookList();
    	iCurrentBook = filename;
   	if (tmp_b!=-1) {
   		if (!iReader->SeekVerse(1, (UInt16)tmp_b,
				(UInt16) tmp_c,(UInt16) tmp_s))	{
			iReader->SeekVerse(1, (UInt16)iReader->GetFirstBook(), 1, 1);
   		}
   	} else {
   		iReader->SeekVerse(1, (UInt16)iReader->GetFirstBook(), 1, 1);
   	}
	TParse p;
	p.SetNoWild(filename, NULL, NULL);
	TPtrC first = p.Name().Left(1);
	if (first==_L("z") || first==_L("Z")) {
		isGreek = ETrue;
		isHebrew = EFalse;
	} else {
		isGreek = EFalse;
		if (first==_L("q") || first==_L("Q")) {
			isHebrew = ETrue;
			Log(_L("Hebrew "));
		} else {
			isHebrew = EFalse;
		}
	}
	

	
//           FillVerse();
      iStartChapter = iReader->ChapterNumber();
      iStartVerse = iReader->VerseNumber();

   	return ETrue;
}

TBool Viewer::ChangeBible(const TDesC &filename)
{
	ChangeBibleL(filename);
	Log(_L("ChangeBible"));	
	return (iReader!=NULL);
}

void Viewer::Init()
{
	iFont = CEikonEnv::Static()->DenseFont();	
	iArea = CEikonEnv::Static()->EikAppUi()->ClientRect();
	CalcScrollLen();
}

int Viewer::GetFontNum()
{
	return iFontNum;
}

void Viewer::setFont(int num)
{
	CWsScreenDevice* dev = CEikonEnv::Static()->ScreenDevice();
	TTypefaceSupport typefaceSupport;
	dev->TypefaceSupport(typefaceSupport, num);
	TInt fontSize = 75;
	//fontSpec = new TFontSpec(typefaceSupport.iTypeface.iName, fontSize);
	//if (fontSpec) {
	//	iFontNum = num;
	//	iCustomFont = CEikonEnv::Static()->CreateScreenFontL(*fontSpec);
	//}
	TFontSpec fontSpec(typefaceSupport.iTypeface.iName, fontSize);
	iFontNum = num;
	if (iCustomFont) {
		CEikonEnv::Static()->ReleaseScreenFont(iCustomFont);	
	}
	iCustomFont = CEikonEnv::Static()->CreateScreenFontL(fontSpec);
	CalcScrollLen();
	if (iLastFont==-1) {
		iLastFont = iFontNum;
		return;
	}
	if (iLastFont!=iFontNum) { 
		ClearVerseList();
		FillVerse();
		iLastFont = iFontNum;
	}
}

const CFont *Viewer::GetFont()
{
	if (iCustomFont==NULL) {
		_LIT(FontName, "LatinBold12");
		TInt fontSize = 75;

		TFontSpec fontSpec(FontName, fontSize);
		iCustomFont = CEikonEnv::Static()->CreateScreenFontL(fontSpec);
		//fontSpec = new TFontSpec(FontName, fontSize);
		//if (fontSpec) {
			//iCustomFont = CEikonEnv::Static()->CreateScreenFontL(*fontSpec);
		//}
	}
	return(iCustomFont==NULL)?iFont:iCustomFont;
}

VerseInfo* Viewer::GetVerseHead()
{
	return this->iVerseHead;
}

void Viewer::AddLast(VerseInfo *v)
{
	if (iVerseHead==NULL) {
		iVerseHead = v;
		iVerseTail = v;
	} else {
		iVerseTail->next = v;
		v->prev = iVerseTail;
		iVerseTail = v;
	}
	iVerseTail->next = NULL;
}

void Viewer::AddFirst(VerseInfo *v)
{
	if (iVerseHead==NULL) {
		iVerseHead = v;
		iVerseTail = v;
	} else {
		v->next = iVerseHead;
		iVerseHead->prev = v;
		iVerseHead = v;
	}
	iVerseHead->prev = NULL;
}


int Viewer::VerseExisted(int chapter, int verse)
{
	VerseInfo *tmp = iVerseHead;
	while (tmp!=NULL) {
		if (tmp->chapter==chapter) {
                  if (tmp->verse==verse) return 1;
                  if (tmp->verse>verse) return 0;
		}
                if (tmp->chapter>chapter) return 0;
                tmp = tmp->next;
	}
	return 0;
}

/**
 * Add at a correct position
 */
void Viewer::Add(VerseInfo *v)
{
	if (iVerseHead==NULL) {
		AddFirst(v);
		return;
	}
	VerseInfo *tmp = iVerseHead;
        //add first
        if (v->chapter<iVerseHead->chapter) {
            AddFirst(v);
            return;
        }
        if (v->chapter==iVerseHead->chapter && v->verse<iVerseHead->verse) {
            AddFirst(v);
            return;
        }
        //add last
        if (v->chapter>iVerseTail->chapter) {
            AddLast(v);
            return;
        }
        if (v->chapter==iVerseTail->chapter && v->verse>iVerseTail->verse) {
            AddLast(v);
            return;
        }
        //add in the middle, list is not empty
	while (tmp!=NULL) {
            if (v->chapter<tmp->chapter || (v->chapter==tmp->chapter && v->verse<tmp->verse)) {
                //insert before tmp
                v->prev = tmp->prev;
                tmp->prev->next = v;
                v->next = tmp;
                tmp->prev = v;
                break;
            }
            tmp = tmp->next;
	}
}

void Viewer::DelFirst()
{
	VerseInfo *tmp;
	if (iVerseHead!=NULL) {
		tmp = iVerseHead;
		iVerseHead=iVerseHead->next;
		if (iVerseHead==NULL) {
			iVerseTail=NULL;
		} else {
			iVerseHead->prev = NULL;
		}
		delete tmp;
	}
}

void Viewer::DelLast()
{
	VerseInfo *tmp, *prev;
	if (iVerseHead==NULL) return;
	tmp = iVerseTail;
	prev = iVerseTail->prev;

	if (prev==NULL) {
		delete iVerseHead;
		iVerseHead = NULL;
		iVerseTail = NULL;
	} else {
		iVerseTail = prev;
		iVerseTail->next = NULL;
		delete tmp;
	}
}



void MyWrapToArrayL  (  const TDesC &    aStringToWrap,  
	const CArrayFix< TInt > &    aLineWidthArray,  
	const CFont &    aFont,  
	CArrayFix< TPtrC > &    aWrappedArray
	
 )  
{
	int i = 0, rest, c;
	int pos;
	CFont::TMeasureTextInput input;
	CFont::TMeasureTextOutput output;
	rest = aStringToWrap.Length();

	pos = 0;
	while (rest>0) {
		input.iMaxBounds = aLineWidthArray[i];
		aFont.MeasureText(aStringToWrap.Mid(pos), &input, &output);
		
		c = output.iChars;
		if (rest - c>0) {
			if (aStringToWrap[c+pos]!=' ') {
				while (c>0 && aStringToWrap[c+pos]!=' ') c--;
				if (c==0) c=1;
			}
		}
		aWrappedArray.AppendL(aStringToWrap.Mid(pos, c));

		rest -= c;
		pos += c;
		i++;
	}
	

}


void Viewer::FillVerse()
{
    int i;
        if (!IsReady()) return;
            TRect drawRect=iArea;
            //if (iArea == TRect(0,0,0,0)) {
	    if (iArea.IsEmpty()) {
                drawRect=CEikonEnv::Static()->EikAppUi()->ClientRect();
            }

            if (iStartChapter == -1) {
                    iStartChapter = iReader->ChapterNumber();
                    iStartVerse = iReader->VerseNumber();
            }

            iReader->SeekVerseInCurrentBook((UInt16)iStartChapter,
                (UInt16)iStartVerse);

            //cache previous verse
            int lim = 10;
            if (iVerseHead==NULL) lim = 5; //if empty, just cache 3 verse
            for(i=1; i<lim; i++) {
                    iReader->MovePrevVerse();
            }
            int width = (TInt)drawRect.Width();
            if (width !=lastWidth) {
				widths->Reset();
              for(i=1; i<50; i++) {
                widths->AppendL(width-5);
              }
              lastWidth = width;
            }

             TBuf<8> strversenum;

            int lines = 0;

        //int cnt = iScrollLen*2 + 5;
        int cnt = 40;
        if (iVerseHead==NULL) cnt = 20; //if empty, just read 20 verse

        for (i=0; i< cnt; i++)
        {
                int lastverse = 0;
                int tv = iReader->VerseNumber();
                int tc = iReader->ChapterNumber();
                if (VerseExisted(tc, tv)) {
                  lastverse = tv;
                  iReader->MoveNextVerse();
                  if (lastverse == iReader->VerseNumber()) break;
                  continue;
                }

                VerseInfo *v = new (ELeave)VerseInfo();
                //v->next = NULL;
                //v->prev = NULL;
                v->verse = tv;
                v->chapter = tc;
                strversenum.Zero();
                strversenum.AppendNum(iReader->VerseNumber());
                int sw = GetFont()->TextWidthInPixels(strversenum)+5;
                widths->At(0) = width-sw;
                if (!iReader->GetCurrentVerseAsString(iTmpBuff)) break;
                //StringToDes(iTmpBuff, iVerseContent);
				if (isGreek) {
					GreekStringToDes(iTmpBuff, iVerseContent);
				} else if (isHebrew) {
					HebrewStringToDes(iTmpBuff, iVerseContent);
				} else {
					MyStringToDes(iTmpBuff, iVerseContent);
				}
                v->vcontent = iVerseContent.AllocL();
                v->lines = new (ELeave)CArrayFixFlat<TPtrC>(30);
                v->lines->Reset();
                MyWrapToArrayL(*(v->vcontent), *widths, *GetFont(), *v->lines);
				
				if (isHebrew) {
					for (int j = 0;j<v->lines->Count(); j++){
						TPtrC s = (v->lines)->At(j);
						TUint16* st = const_cast<TUint16*>(s.Ptr());
						for (int k = 0; k<s.Length()/2; k++) {
								TUint16 tmp = st[k];
								st[k] = st[s.Length()-k-1];
								st[s.Length()-k-1] = tmp;
						}
						
					}				
				}
				
				
                lines += v->lines->Count();
/*
                iTmpBuff[0] = 0;
                iReader->GetCurrentTextAsString(iTmpBuff, Decoder::bookTextType);
                if (iTmpBuff[0]) {
                        MyStringToDes(iTmpBuff, iVerseContent);
                        v->booktext = iVerseContent.AllocL();
                        v->booklines = new (ELeave)CArrayFixFlat<TPtrC>(7);
                        v->booklines->Reset();
                        AknTextUtils::WrapToArrayL(*(v->booktext), *widths, *GetFont(), *v->booklines);
                }
                iTmpBuff[0] = 0;
                iReader->GetCurrentTextAsString(iTmpBuff, Decoder::chapTextType);

                if (iTmpBuff[0]) {
                        MyStringToDes(iTmpBuff, iVerseContent);
                        v->chaptext = iVerseContent.AllocL();
                        v->chaplines = new (ELeave)CArrayFixFlat<TPtrC>(20);
                        v->chaplines->Reset();
                        AknTextUtils::WrapToArrayL(*(v->chaptext), *widths, *GetFont(), *v->chaplines);
                }

                iTmpBuff[0] = 0;
                iReader->GetCurrentTextAsString(iTmpBuff, Decoder::descTextType);
                if (iTmpBuff[0]) {
                        MyStringToDes(iTmpBuff, iVerseContent);
                        v->desctext = iVerseContent.AllocL();
                        v->desclines = new (ELeave)CArrayFixFlat<TPtrC>(20);
                        v->desclines->Reset();
                        AknTextUtils::WrapToArrayL(*(v->desctext), *widths, *GetFont(), *v->desclines);
                }
*/
                Add(v);
                lastverse = v->verse;
                iReader->MoveNextVerse();
                if (lastverse == iReader->VerseNumber()) break;
                //if (lines>40) break;
        }
#define MAX_VERSE_IN_MEM 50
#define VERSE_LIMIT (MAX_VERSE_IN_MEM/2)
        VerseInfo *v = GetVerseHead();
        int listcount = 0;
        int position = 0;
        while (v) {
                if (v->chapter==iStartChapter && v->verse==iStartVerse) {
                    position = listcount;
                }
                v = v->next;
                listcount++;
        }
        if (listcount>MAX_VERSE_IN_MEM) {
          if (position>VERSE_LIMIT) {
            int c = position - VERSE_LIMIT;
            while (c--) { DelFirst();}
          }
          if (listcount-position>VERSE_LIMIT) {
            int c = (listcount-position)-VERSE_LIMIT;
            while (c--) { DelLast();}
          }
        }
}

void Viewer::ClearVerse()
{
	while (iVerseHead) DelFirst();
	iStartVerse = -1;
	iStartChapter = -1;
	iStartLine = 0;
}

void Viewer::ClearVerseList()
{
	while (iVerseHead) DelFirst();
}


void Viewer::UpdateBuffer()
{
	VerseInfo *v = iVerseHead;
	int line_count = 0;
	while (v) {
		if (v->chapter==iStartChapter && v->verse==iStartVerse) {
			break;
		}
		v = v->next;
	}
	while (v) {
		line_count += v->lines->Count();
		v = v->next;
	}
	if (line_count < (iScrollLen)) FillVerse();
}

void Viewer::ScrollDown(int count)
{
	if (!IsReady()) return;
	VerseInfo *v = iVerseHead;

restart_up_search:
	while (v) {
		if (v->verse==iStartVerse && v->chapter==iStartChapter) {
			break;
		}
		v = v->next;
	}

	//scroll in one verse
	if (count<v->lines->Count()-iStartLine) {
		iStartLine+=count;
		return;
	}

	count -= (v->lines->Count()-iStartLine);


	iStartLine = 0;

	v=v->next;

	if (!v) { //end of buffer reached
		iReader->SeekVerseInCurrentBook((UInt16)iStartChapter,
			(UInt16)iStartVerse);
		iReader->MoveNextVerse();
		FillVerse(); //refill buffer
		v = iVerseHead; //restart search
		while (v) {
			if (v->verse==iStartVerse && v->chapter==iStartChapter) {
				break;
			}
			v = v->next;
		}
		v = v->next;
	}

	if (!v) return; //useless, end of book reached
	iStartVerse = v->verse;
	iStartChapter = v->chapter;

	while (v && count>0) {
		iStartVerse = v->verse;
		iStartChapter = v->chapter;

		if (v->lines->Count()>count) {
			iStartLine = count;
			count = 0;
			break;
		} else {
			count -= v->lines->Count();
		}
		v = v->next;
		if (v && count == 0) {
			iStartVerse = v->verse;
			iStartChapter = v->chapter;
		}

	}
	if (count>0) { //long scroll
		FillVerse(); //refill buffer
		v = iVerseHead; //restart search
		goto restart_up_search;
	}
}


void Viewer::SaveState()
{
	tmpStartVerse = iStartVerse;
	tmpStartChapter = iStartChapter;
}

void Viewer::LoadState()
{
	iStartVerse = tmpStartVerse;
	iStartChapter = tmpStartChapter;
}



void Viewer::ScrollUp(int count)
{
	if (!IsReady()) return;
	VerseInfo *v = iVerseHead;

restart_down_search:
	while (v) {
		if (v->verse==iStartVerse && v->chapter==iStartChapter) {
			break;
		}
		v = v->next;
	}
	//scroll in one verse
	if (count<=iStartLine) {
		iStartLine -= count;
		return;
	}

	v = v->prev;
	if (!v) { //start of buffer reached
		iReader->SeekVerseInCurrentBook((UInt16)iStartChapter,
			(UInt16)iStartVerse);
		iReader->MovePrevVerse();
		FillVerse(); //refill buffer
		v = iVerseHead; //restart search
		while (v) {
			if (v->verse==iStartVerse && v->chapter==iStartChapter) {
				break;
			}
			v = v->next;
		}
		v = v->prev;
	}
	iStartLine = 0;
	if (!v) return; //useless, start of book reached
	while (v && count>0) {
		iStartVerse = v->verse;
		iStartChapter = v->chapter;
		if (v->lines->Count()>count) {
			iStartLine = v->lines->Count()-count;
			count = 0;
			break;
		} else {
			count -= v->lines->Count();
		}
		v = v->prev;
		if (v && count == 0) {
			iStartVerse = v->verse;
			iStartChapter = v->chapter;
		}
	}
	if (count>0) { //long scroll
		FillVerse(); //refill buffer
		v = iVerseHead; //restart search
		goto restart_down_search;
	}
}

void Viewer::NextVerse()
{
	if (!IsReady()) return;

	VerseInfo *v = iVerseHead;

	while (v) {
		if (v->chapter==iStartChapter && v->verse==iStartVerse) {
			break;
		}
		v = v->next;
	}
	if (v && v->next) {
		v = v->next;
		iStartChapter = v->chapter;
		iStartVerse = v->verse;
		iStartLine = 0;
		return;
	}
	//end of buffer reached
	iReader->SeekVerseInCurrentBook((UInt16)iStartChapter, (UInt16)iStartVerse);
	iReader->MoveNextVerse();
	FillVerse(); //refill buffer
	v = iVerseHead; //restart search
	while (v) {
		if (v->verse==iStartVerse && v->chapter==iStartChapter) {
			break;
		}
		v = v->next;
	}
	if (v && v->next) {
		v = v->next;
		iStartChapter = v->chapter;
		iStartVerse = v->verse;
		return;
	}

}

void Viewer::PrevVerse()
{
	if (!IsReady()) return;

	VerseInfo *v = iVerseHead;

	while (v) {
		if (v->verse==iStartVerse && v->chapter==iStartChapter) {
			break;
		}
		v = v->next;
	}
	if (v && v->prev) {
		v = v->prev;
		iStartChapter = v->chapter;
		iStartVerse = v->verse;
		iStartLine = 0;
		return;
	}
	//end of buffer reached
	iReader->SeekVerseInCurrentBook((UInt16)iStartChapter, (UInt16)iStartVerse);
	iReader->MoveNextVerse();
	FillVerse(); //refill buffer
	v = iVerseHead; //restart search
	while (v) {
		if (v->verse==iStartVerse && v->chapter==iStartChapter) {
			break;
		}
		v = v->next;
	}
	if (v && v->prev) {
		v = v->prev;
		iStartChapter = v->chapter;
		iStartVerse = v->verse;
		return;
	}
}

void Viewer::PageDown()
{
	ScrollDown(iScrollLen);
	UpdateBuffer();
}

void Viewer::GotoTop()
{
	iStartChapter = 1;
	iStartVerse = 1;
	iStartLine = 0;
//        ClearVerse();
	//ClearVerseList();
	UpdateBuffer();
}

void Viewer::GotoBottom()
{
	if (!IsReady()) return ;
//        ClearVerse();
	iReader->GetLastVerse(&iStartChapter,  &iStartVerse);
	iStartLine = 0;
	//ClearVerseList();
	UpdateBuffer();
}

void Viewer::NextChapter()
{
	if (!IsReady()) return ;
        int oldChapter = iStartChapter;
	iStartChapter++;

	if (iStartChapter>iReader->getTotalChapter()) {
		iStartChapter = iReader->getTotalChapter();
	}
        if (oldChapter==iStartChapter) return;
        ClearVerseList();
        iStartVerse = 1;
	iStartLine = 0;
	iReader->SeekVerseInCurrentBook((UInt16)iStartChapter,
                (UInt16)iStartVerse);
	FillVerse();
}

void Viewer::PrevChapter()
{
	if (!IsReady()) return ;
        if (iStartChapter==1) {
          return;
        }
	int tmp = iStartChapter;
        ClearVerseList();
	iStartChapter = tmp;
	iStartChapter--;
	iStartVerse = 1;
	iStartLine = 0;
	iReader->SeekVerseInCurrentBook((UInt16)iStartChapter,
                (UInt16)iStartVerse);
	FillVerse();
}

void Viewer::PageUp()
{
	ScrollUp(iScrollLen);
	UpdateBuffer();
}

char *Viewer::BookName()
{
	return iReader->BookName();
}

char* Viewer::SimpleBookName()
{
	return iReader->SimpleBookName();
}

bool Viewer::SeekVerse(UInt16 version, UInt16 book, UInt16
				chapter, UInt16 verse)
{
	if (!IsReady()) return false;
	return iReader->SeekVerse(version, book, chapter, verse);
}

bool Viewer::SeekVerseAsString(UInt16 version, char *str)
{
	if (!IsReady()) return false;
	return iReader->SeekVerseAsString(version, str);
}

void Viewer::NextBook()
{
	if (!IsReady()) return;
	if (iReader->GetNextBook()!=-1) {
		ClearVerse();
		SeekVerse(1, (UInt16)iReader->GetNextBook(), 1, 1);
		FillVerse();
	}
}

void Viewer::PrevBook()
{
	if (!IsReady()) return;
	if (iReader->GetPrevBook()!=-1) {
		iStartChapter = 1;
		iStartVerse = 1;
		iStartLine = 0;
		ClearVerse();
		SeekVerse(1, (UInt16)iReader->GetPrevBook(), 1, 1);
		FillVerse();
	}
}

int Viewer::GetStartChapter()
{
	return iStartChapter;
}

int Viewer::GetStartVerse()
{
	return iStartVerse;
}

int Viewer::GetStartLine()
{
	return iStartLine;
}

int Viewer::GetBookNumber()
{
    if (!IsReady()) return 0;
    return iReader->BookNumber();
}

int Viewer::IsReady()
{
	return iReader!=NULL;
}

void Viewer::GotoBook(int book)
{
	if (!IsReady()) return;
	iReader->SeekVerse(1, (UInt16)book, 1, 1);
	ClearVerse();
	FillVerse();
}

void Viewer::GotoBookByIndex(int idx)
{
	if (!IsReady()) return;
	iReader->SeekVerse(1, (UInt16)iBookNumberList[idx], 1, 1);
	ClearVerse();
	FillVerse();
}


TInt& Viewer::GetBinding(int idx)
{
	return iBinding[idx];
}


void Viewer::LoadBindings()
{
    int i;
	CStringList *settings = CStringList::NewLC(CEikonEnv::Static()->FsSession());
	settings->LoadFromFileL(iBindingsFile);
	Mem::FillZ(iBinding, sizeof(iBinding));
	for (i = 0; i<settings->Count();i++){
		iBinding[i] =settings->GetNumberAt(i);
	}
	CleanupStack::PopAndDestroy(settings);
}

void Viewer::SaveBindings()
{
	CStringList *settings = CStringList::NewLC(CEikonEnv::Static()->FsSession());
	for (int i  = 0; i<15; i++) {
		settings->AddNumberL(iBinding[i]);
	}
	settings->SaveToFileL(iBindingsFile);
	CleanupStack::PopAndDestroy(settings);
}

/* Converts a C string to a descriptor
 * taken from the putty source code
 */
void Viewer::StringToDes(const char *aStr, TDes &aTarget) {

    // Doesn't really work for anything except US-ASCII
    // Copied from putty
    // Used as fallback if characterconverter is not available
    aTarget.SetLength(0);
    while ( *aStr ) {
        aTarget.Append(*aStr++);
    }
}

void Viewer::SetEncoding(TUint Encoding) {
	CurrentEncoding = Encoding;
	//fist time
	if (iLastEnc==-1) {
		iLastEnc = CurrentEncoding;
		return;
	}
	if ((TUint)iLastEnc!=CurrentEncoding) {
		delete iCharacterConverter;
		iCharacterConverter = CCnvCharacterSetConverter::NewLC();
		iConverterAvailability = iCharacterConverter->PrepareToConvertToOrFromL(CurrentEncoding, iFs);
		CleanupStack::Pop(iCharacterConverter);
		ClearVerse();
		FillVerse();
		iLastEnc = CurrentEncoding;
	}
}

TUint Viewer::GetEncoding()
{
	return CurrentEncoding;
}

/* Converts a descriptor into a C string. The string needs to be pre-allocated
 * taken from the putty source code
 */
void Viewer::DesToString(const TDesC &aDes, char *aTarget) {
    int i = 0;
    int len = aDes.Length();
    while ( i < len ) {
        *aTarget = (char) aDes[i];
        aTarget++;
        i++;
    }
    *aTarget = 0;
}


void Viewer::MyStringToDes(const char *aStr, TDes &aTarget)
{
      TInt aState = CCnvCharacterSetConverter::KStateDefault;
      if (iConverterAvailability == CCnvCharacterSetConverter::EAvailable) {
	TPtrC8 tmp((TUint8*)aStr, User::StringLength((const TUint8*)aStr));
        iCharacterConverter->ConvertToUnicode(aTarget, tmp, aState);
      } else {
          StringToDes(aStr, aTarget);
      }
}


void Viewer::GreekStringToDes(const char *aStr, TDes &aTarget)
{
    aTarget.SetLength(0);
    while ( *aStr ) {
        aTarget.Append(greektab[*aStr]);
		aStr++;
    }
}


void Viewer::HebrewStringToDes(const char *aStr, TDes &aTarget)
{
    aTarget.SetLength(0);
    while ( *aStr ) {
        aTarget.Append(hebrewtab[*aStr]);
		aStr++;
    }
}


void Viewer::MyDesToString(const TDesC &aDes, char *aTarget)
{
      if (iConverterAvailability == CCnvCharacterSetConverter::EAvailable) {
        TBuf<1> unicodeBuffer;
        TBuf8<10> asciiBuffer ;
        for (int i = 0; i<aDes.Length(); i++){
            unicodeBuffer.SetLength(0);
            unicodeBuffer.Append(aDes[i]);
            iCharacterConverter->ConvertFromUnicode(asciiBuffer, unicodeBuffer);
            int j = 0;
            int len = asciiBuffer.Length();
            while (j < len) {
              *aTarget = (char)asciiBuffer[j];
              aTarget++;
              j++;
            }
        }
        *aTarget = 0;
      } else {
          DesToString(aDes, aTarget);
      }
}

TBool Viewer::IsRToL()
{
	return isHebrew;
}