/*
    S60Bible A Bible Reader for SymbianOS
    Copyright (C) 2004 Yohanes Nugroho

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Yohanes Nugroho (yohanes@opensource.or.id)
    Kp Areman RT 09/08 No 71 Desa Tugu Cimanggis Depok 16951 Indonesia
*/


#include <coemain.h>
#include <eikenv.h>
#include <e32keys.h>
#include <aknutils.h>
#include <e32std.h>
#include <aknnotewrappers.h>
#include <aknviewappui.h>
#include <eiktxlbx.h>
#include <aknpopup.h>
#include <aknlists.h>

#include "BookListMainView.h"
#include "S60BibleDocument.h"
#include "S60Bible.hrh"
#include "S60Bible.pan"
#include <S60Bible.rsg>
#include "S60BibleUid.h"
#include "BookListContainer.h"
#include "log.h"

CBookListMainView* CBookListMainView::NewL(const TRect& aRect, CS60BibleDocument* aDocument)
{
	CBookListMainView* self = CBookListMainView::NewLC(aRect, aDocument);
	CleanupStack::Pop(self);
	return self;
}

TUid CBookListMainView::Id() const
{
	return TUid::Uid(KBookListViewId);
}

CBookListMainView* CBookListMainView::NewLC(const TRect& aRect, CS60BibleDocument* aDocument)
{
	CBookListMainView* self = new (ELeave) CBookListMainView();
	CleanupStack::PushL(self);
	self->iDocument = aDocument;
	self->ConstructL(aRect);
	return self;
}

CBookListMainView::CBookListMainView()
{
	
}


CBookListMainView::~CBookListMainView()
{
    //  DoDeactivate is called by the base class destructor
}

void CBookListMainView::ConstructL(const TRect& /*aRect*/)
{
	BaseConstructL(R_S60BIBLE_BOOKLISTVIEW);
}

void  CBookListMainView::DialogDismissedL (  TInt    /*aButtonId*/  )
{

}

void CBookListMainView::DoActivateL(const TVwsViewId& aPrevViewId,
                                    TUid /*aCustomMessageId*/,
                                    const TDesC8& /*aCustomMessage*/)
{
	ASSERT(iContainer == NULL);
	iContainer = CBookListContainer::NewL(this, iDocument);
	iRetVal = TUid::Uid(0);
	AppUi()->AddToStackL(iContainer);
	iPrevView = aPrevViewId;
}

void CBookListMainView::DoDeactivate()
{
	if (iContainer) {
		AppUi()->RemoveFromStack(iContainer);
		delete iContainer;
		iContainer = NULL;
		iDocument->Model()->ClearBookList();
		User::CompressAllHeaps();
	}
}

void CBookListMainView::SwitchBack()
{
	AppUi()->ActivateLocalViewL(iPrevView.iViewUid, iRetVal, _L8(""));
}

void CBookListMainView::SetRetVal(int i)
{
	iRetVal = TUid::Uid(i);
}

void CBookListMainView::HandleCommandL(TInt aCommand)
{
	switch(aCommand) {
		case KGotoBack:
		case EAknSoftkeyBack:
			{
				SwitchBack();
				break;
			}
		case KGotoBook:
			{
				SetRetVal(1);
				iContainer->GotoBook();
			}

		default:
			//User::Panic(KS60BiblePanicName, ES60BibleUnknownCommand);
			break;
	}

}
