/*
    S60Bible A Bible Reader for SymbianOS
    Copyright (C) 2004 Yohanes Nugroho

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Yohanes Nugroho (yohanes@opensource.or.id)
    Kp Areman RT 09/08 No 71 Desa Tugu Cimanggis Depok 16951 Indonesia
*/


/*
 * Filename: S60BibleAppUi.cpp
 * Description: Bible user interface class
 * Author: Yohanes Nugroho
 * Creation Date:
 * Last Update: 15/Feb/2004
 * Revisions:
 */
 
#include <e32std.h>
#include <S60Bible.rsg>
#include <eiktbar.h>
#include <eikenv.h>
#include <eiknotapi.h>
#include <w32adll.h>
#include <aknnotedialog.h>
#include <aknwaitdialog.h>
#include <aknnotewrappers.h>
#include <aknglobalnote.h>
#include <avkon.hrh>
#include <aknlists.h>

#include "S60Bible.pan"
#include "S60BibleDocument.h"
#include "S60BibleAppUi.h"
#include "S60BibleMainView.h"
#include "BookListMainView.h"
#include "BookmarkView.h"
#include "BibleSettingListView.h"
#include "KeyBindingView.h"
#include "Abouts60.h"
#include "S60Bible.hrh"


CS60BibleAppUi::CS60BibleAppUi(CS60BibleDocument* aDocument)
:	iDocument(aDocument)
{
	//	No implementation required
}


void CS60BibleAppUi::ConstructL()
{
#ifdef __SERIES60_30__
	BaseConstructL(EAknEnableSkin);
#else
	BaseConstructL();
#endif

	// Enable the virtual cursor
	//iEikonEnv->VirtualCursor().SetCursorStateL(TEikVirtualCursor::EOn,*iEikonEnv);

	iMainView = CS60BibleMainView::NewL(ClientRect(), iDocument);
	iBookmarkView = CBookmarkView::NewL(ClientRect(), iDocument);
	iBookListView = CBookListMainView::NewL(ClientRect(), iDocument);
	iKeyBindingView = CKeyBindingView::NewL(ClientRect(), iDocument);

	iSettingView = CBibleSettingListView::NewL(ClientRect(), iDocument);

	iAboutView = CAboutView::NewL();

	AddViewL(iMainView); // transfer ownership to base class
	AddViewL(iBookmarkView); // transfer ownership to base class
	AddViewL(iBookListView); // transfer ownership to base class
    AddViewL(iAboutView);
	AddViewL(iSettingView);
	AddViewL(iKeyBindingView);
	SetDefaultViewL(*iMainView);
}

CS60BibleAppUi::~CS60BibleAppUi()
{
}


void CS60BibleAppUi::HandleCommandL(TInt aCommand)
{
	switch(aCommand)
	{
		case EEikCmdExit:
		case EAknSoftkeyExit:
			Exit();
			break;
	}
}

#ifdef __SERIES60_30__
void CS60BibleAppUi::HandleResourceChangeL(TInt aType)
{
	CAknAppUi::HandleResourceChangeL( aType ); 
    if  ( aType == KEikDynamicLayoutVariantSwitch ) 
    { 
    	iSettingView->SizeChanged();
    	iKeyBindingView->SizeChanged();
    }   
}
#endif

void CS60BibleAppUi::HandleForegroundEventL(TBool aForeground)
{
	if(!aForeground) {
		//lost focus
		iMainView->LostFocus();
	} else {
		//gained focus
		iMainView->GainFocus();
	}

	CAknAppUi::HandleForegroundEventL(aForeground);
}

