/*
    S60Bible A Bible Reader for SymbianOS
    Copyright (C) 2004 Yohanes Nugroho

    Original work (PalmBible and Bible+):
    Copyright (C) 2001  Ho Chuen Poon
    Modifications (C) 2003 Yih-Chun Hu

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Yohanes Nugroho (yohanes@opensource.or.id)
    Kp Areman RT 09/08 No 71 Desa Tugu Cimanggis Depok 16951 Indonesia
*/


#ifndef DECODER_H
#define DECODER_H
#include "Database.h"
#include "Verse.h"
#include "FullReader.h"
#include "info.h"
#include "VersionInfo.h"

/*
#define bookTextType 0xFFFF
#define chapTextType 0xFFFE
#define descTextType 0xFFFD
#define versTextType 0xFFFC
//#define searchBufferSize 192

*/
#define searchBufferSize 16384

//extern bool dualView;
//extern bool noBlankDual;
//extern bool noBlankNormal;
//extern bool vBold;
//extern UInt16 wordSpace;

typedef struct WordIndexType {
	UInt16 wordLength;
	UInt16 totalWord;
	bool compressed;
	bool nothing;
} WordIndexType;
typedef WordIndexType *WordIndexPtr;

typedef struct _WordInfoType {
	UInt16 totalIndexes;
	WordIndexType *wordIndex;
} WordInfoType;
typedef WordInfoType *WordInfoPtr;

class Decoder {
      public:
  enum {
    bookTextType=0xFFFF,
    chapTextType=0xFFFE,
    descTextType=0xFFFD,
    versTextType=0xFFFC
  } ETextType;

	Decoder(VersionInfo *vinfo, UInt16 version);
	void ConstructL();
	~Decoder();
	void DecodeVerseType(char *verse,
				 FullReader * recordReader,
			     UInt32 verseStart, UInt16 verseLength, int type);
	void DecodeVerseAsString(char *verse,
                                 FullReader * recordReader,
                                 UInt32 verseStart, UInt16 verseLength);
	void DecodeVerseTypeF(char *verse, UInt16 *linepos,
			       UInt16 *lineformat, int *linecount,
			       FullReader * recordReader,
			      UInt32 verseStart, UInt16 verseLength);
      private:
	Database *dbase;
	UInt16 getNumWidth(UInt16 value);
	void getWordIndexLen();
	void beginVerse();
	UInt16 readVerse();
	void endVerse();
	void beginWord();
	void readWord();
	void endWord();
	void checkCompressedWordNum();
	void readCompressed();

	VersionInfo *bibleversion;
	FullReader *decWordReader;
	FullReader *decVerseReader;
	WordInfoPtr decWordInfo;

	UInt16 version;
	UInt16 recordIndex;
	UInt16 totalRecord;

	WordInfoPtr wordInfo;
	FullReader *wordReader;
	char sepChar;
	bool byteShifted;

	UInt16 decScreenPos, decLinePos, decCharPos;
	bool decTyped, decBlankAfter;

	char decCharBuffer[64];
	//char *decCharBuffer;
	
	char *decLineBuffer;
	char *decVerseBuffer[16384];

	UInt16 decCompressedBuffer[32];

	bool decCompressed;
	bool decVBoldEnable;

	char decSepChar;
	char decLastType;

	UInt32 decVerseStart;
	UInt16 decShift;
	UInt32 decValueBuffer[3];

	UInt32 decWordIndex;
	UInt16 decWordNum, decWordLen;
	UInt16 decRepeat, decCompressedIndex;
	UInt16 decByteShifted;
	UInt32 *byteacc;
};
#endif

