/*
    S60Bible A Bible Reader for SymbianOS
    Copyright (C) 2004 Yohanes Nugroho

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Yohanes Nugroho (yohanes@opensource.or.id)
    Kp Areman RT 09/08 No 71 Desa Tugu Cimanggis Depok 16951 Indonesia
*/

#ifndef __S60BIBLEDOCUMENT_H__
#define __S60BIBLEDOCUMENT_H__


#include <eikdoc.h>
#include <errorui.h>

#include "Viewer.h"
#include "CFileFinderEngine.h"
#include <aknwaitdialog.h>


// Forward references
class CS60BibleAppUi;
class CEikApplication;

/*!
  @class CS60BibleDocument
  @discussion An instance of class CS60BibleDocument is the Document part of the CKON
  application framework for the S60Bible example application
  */
class CS60BibleDocument : public CEikDocument
    {
public: // New methods


/*!
  @function NewLC
  @abstract Constructs a CS60BibleDocument for the application aApp,
  and return a pointer to the created object
  @result Returns a pointer to the instance which is also on the cleanup stack
  */
    static CS60BibleDocument* NewLC(CEikApplication& aApp);
/*!
  @function NewL
  @abstract Constructs a CS60BibleDocument for the application aApp,
  and return a pointer to the created object
  @result Returns a pointer to the instance
  */
    static CS60BibleDocument* NewL(CEikApplication& aApp);

/*!
  @function ~CS60BibleDocument
  @abstract Destroys the object and release all memory objects
  */
    ~CS60BibleDocument();

/*!
  @function Model
  @abstract Gets the model associated with the document
  @result Returns a reference to a pointer to the model
  */
     Viewer * Model();


private: // New methods

/*!
  @function CS60BibleDocument
  @abstract Performs the first phase of two phase construction
  @param aApp the application
  */
    CS60BibleDocument(CEikApplication& aApp);

/*!
  @function ConstructL
  @abstract Performs the second phase construction of a CS60BibleDocument object
  */
    void ConstructL();

/*!
  @function CreateAppUiL
  @abstract Creates a CS60BibleAppUi object and return a pointer to it
  @result Returns a pointer to the Application's User Interface object
  */
    CEikAppUi* CreateAppUiL();


private:


	/*! @var iModel The model	*/
	/*CShapeListManager* iModel;*/
	Viewer *iViewer;
	CErrorUI*         iErrorUI;
	CFileFinderEngine* iFileFinder;
	CAknWaitDialog* waitDialog;

	};


#endif // __S60BibleDOCUMENT_H__
